
"""
Copyright 2006 Edward C. Jones

This file is part of mrquery.

mrquery is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

mrquery is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
mrquery; if not, write to the Free Software Foundation, Inc., 51 Franklin
St, Fifth Floor, Boston, MA 02110-1301 USA
"""
import sys
import numarray
from numarray.numerictypes import *
from LN import LN, MLLN

types = (None, Bool, Int8, UInt8, Int16, UInt16, Int32, UInt32,
            Int64, UInt64, Float32, Float64)

inttypes = (Bool, Int8, UInt8, Int16, UInt16, Int32, UInt32, Int64, UInt64)

names = ('Any', 'Bool', 'Int8', 'UInt8', 'Int16', 'UInt16', 'Int32',
         'UInt32', 'Int64', 'UInt64', 'Float32', 'Float64')

# On my machine,
#    b = FLT_RADIX = 2
#    float:  p = FLT_MANT_DIG = 24
#            emax = FLT_MAX_EXP = 128
#    double: p = DBL_MANT_DIG = 53
#            emax = DBL_MAX_EXP = 1024
#    MAX = (1 - b**-p) * b**emax = (b**p - 1) * b**(emax-p)
#    FLT_MAX = 0x1.fffffep+127 = (2**24 - 1) * 2**104
#    DBL_MAX = 0x1.fffffffffffffp+1023 = (2**53 - 1) * 2**971
# See "float.h".
# It is ok that FLT_MAX is a Python float (which is a C double).
FLT_MAX = float((2**24 - 1) * 2**104)
DBL_MAX = float((2**53 - 1) * 2**971)
mins = (None, 0, -2**7, 0, -2**15, 0, -2**31, 0, -2**63, 0,
          -FLT_MAX, -DBL_MAX)

maxs = (None, 1, 2**7-1, 2**8-1, 2**15-1, 2**16-1, 2**31-1, 2**32-1,
        2**63-1, 2**64-1, FLT_MAX, DBL_MAX)

limits = dict()
for i in range(1, 12):
    limits[types[i]] = (mins[i], maxs[i])

def clipcast(arr, typeout):
    """Convert an array to another type. The array is clipped.

    Hopefully, there is no garbage in the output due to C casts.

    This code contains workarounds for some bugs in numarray.
    """
    typein = arr.type()
    temp = numarray.array([], typeout)
    typeout = temp.type()

    inmin = limits[typein][0]
    inmax = limits[typein][1]
    outmin = limits[typeout][0]
    outmax = limits[typeout][1]
    if arr.type() == typeout:
        return arr.copy()
    elif arr.type() in inttypes and typeout in inttypes:
        themin = max(inmin, outmin)
        themax = min(inmax, outmax)
        temparr = numarray.clip(arr, themin, themax)
        return temparr.astype(typeout)
    else:
        # Parts of "temparr" will be garbage. Those parts aren't used.
        numarray.Error.pushMode(invalid='ignore', overflow='ignore')
        temparr = arr.astype(typeout)
        numarray.Error.popMode()
        minarr = numarray.array([outmin], typeout)
        temparr = numarray.where(arr > minarr, temparr, outmin)
        maxarr = numarray.array([outmax], typeout)
        out = numarray.where(arr < maxarr, temparr, outmax)
        return out

def check():
    for i in range(1, 12):
        for j in range(1, 12):
            inmin = mins[i]
            inmax = maxs[i]
            outmin = mins[j]
            outmax = maxs[j]
            themin = max(inmin, outmin)
            themax = min(inmax, outmax)
            print types[i], types[j]
            try:
                if types[i] in (Float32, Float64):
                    arrin = numarray.array(
                          [inmin, float(themin), float(themax), inmax],
                          Float64)
                    if types[i] == Float32:
                        arrin = arrin.astype(Float32)
                else:
                    arrin = numarray.array(
                          [inmin, themin, themax, inmax], types[i])
                print 'IN ', arrin
                arrout = clip(arrin, types[j])
                print 'OUT', arrout
                if arrin[2] != arrout[2]:
                    # There are
                    # 1. Off-by-one errors which are due to floating
                    #    precision issues. For these the last two numbers
                    #    printed are "0.0 1.0" or "0.0 -1.0".
                    # 2. Errors that aren't really errors. For these the
                    #    last two numbers printed are "0.0 0.0".
                    print 'ERROR', arrin[2], arrout[2], (arrout - arrin)[2], \
                          arrout[2] - arrin[2]
            except Exception, s:
                print 'EXCEPTION', s.__class__, s
            print
