#! /usr/bin/env python

"""
Copyright 2006 Edward C. Jones

This file is part of mrquery.

mrquery is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

mrquery is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
mrquery; if not, write to the Free Software Foundation, Inc., 51 Franklin
St, Fifth Floor, Boston, MA 02110-1301 USA
"""
import os, sys, optparse
import utilities

def countpics(topdirs, sortit=False):
    """Count the number of images in each directory under the topdirs.

    "topdirs" can be a directory name or a list of directory names.
    The output is sent to stdout.
    """
    if isinstance(topdirs, str):
        topdirs = (topdirs,)
    for topdir in topdirs:
        counts = dict()
        topdir = os.path.abspath(topdir)
        for name in utilities.WalkImages(topdir):
            end = topdir.rindex('/')
            virtual_name = name[end:]
            end = name
            parts = virtual_name.split('/')[1:-1]
            for i in range(len(parts)):
                adir = os.path.join(*parts[:i+1])
                if adir not in counts:
                    counts[adir] = 0
                counts[adir] += 1
        keys = counts.keys()
        keys.sort()
        out = list()
        for key in keys:
            if sortit or '/' not in key:
                name = key
            else:
                i = key.rindex('/')
                name = key[i+1:]
            spaces = ''
            if not sortit:
                spaces = '    ' * key.count('/')
            out.append('%6i %s%s' % (counts[key], spaces,  name))
        if sortit:
            out.sort(reverse=True)
        print topdir
        for s in out:
            print s
        print

if __name__ == '__main__':
    usage = 'USAGE: ./countpics [OPTIONS] <topdir> [<topdir> ...]'
    parser = optparse.OptionParser(usage=usage)
    parser.add_option('-s', '--sort', default=False,
         action='store_true', help='sort the output by image count')
    options, args = parser.parse_args()

    countpics(args, options.sort)
