#! /usr/bin/env python

"""Print text for changeing the list of directories to be ignored when creating html"""

"""
Copyright 2006 Edward C. Jones

This file is part of mrquery.

mrquery is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

mrquery is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
mrquery; if not, write to the Free Software Foundation, Inc., 51 Franklin
St, Fifth Floor, Boston, MA 02110-1301 USA
"""
"""
Sometimes there are directories with good image matches, that you already
know about, don't plan to change, and don't want to see in the html output.
A list of these directories is kept in "mrq.html_ignores". These directories
are called "ignored". The rule is: If both elements of a directory pair
(dirpair) are under a directory in mrq.html_ignores, no html is produced for
the pair.

This program prints a list of all the directories under the topdirs for a
set of mrq data. The currently ignored directories have an "X" in front of
them. Edit the output of this program, putting "X"s in front of directories
that you want ignored. If an "X" is before a directory name then there is
no need to put "X"s in front of any subdirectories. They are automatically
ignored.

Note that the first three lines of the output look like:

    basename
    X Do nothing.
      Create html for all directory pairs (reinitialize mrq.html_ignores).

If the "X" is left in front of "Do nothing.", nothing will be changed. An
"X" before the "Create html" will cause all the ignores to be removed.

When you are done editing the file, run
    ./setignores.py filename
"""
import os, sys, textwrap, optparse
import constants, pathnames, utilities, mrquery
from LN import LN, MLLN

usage = """\
USAGE: ./getXdirs.py [-r] <basename>
       The output should be redirected into a file."""
parser = optparse.OptionParser(usage=usage)
parser.add_option('-r', '--reinitialize', default=False, action='store_true',
    help='resets data so no directories are ignored')
options, args = parser.parse_args()
if len(args) != 1:
    parser.print_help()
    sys.exit('requires one argument')

topdatadir = constants.TOPDATADIR
basename = args[0]

mrq = mrquery.Retrieve_MRQuery(constants.TOPDATADIR, basename)
if options.reinitialize:
    mrq.html_ignores = list()
ignores = set(mrq.html_ignores)

topXtextDelete_piece = """\
    X Do nothing
      Create html for all directory pairs (reinitialize mrq.html_ignores).
"""
topXtextDelete_piece = textwrap.dedent(topXtextDelete_piece)

print basename
print topXtextDelete_piece
for dirname in mrq.alldirs:
    if dirname in ignores:
        print 'X %s' % dirname
    else:
        print '  %s' % dirname
mrq.Store()
