#! /usr/bin/env python

"""Useful way to run the mrquery programs"""

"""
Copyright 2006 Edward C. Jones

This file is part of mrquery.

mrquery is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

mrquery is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
mrquery; if not, write to the Free Software Foundation, Inc., 51 Franklin
St, Fifth Floor, Boston, MA 02110-1301 USA
"""
import os, sys, optparse
import constants

class TopLevel(object):
    def __init__(self, basename, topdirs=None):
        # The following must be called as early as possible when running the
        # mrquery family of programs.
        os.stat_float_times(True)
        self.basename = basename
        self.topdirs = topdirs
        self.topdatadir = constants.TOPDATADIR

    def do_mrquery(self, reuse_MRQuery, excludes=None):
        import mrquery
        print >> sys.stderr, 'MRQuery'
        if excludes is None:
            excludes = list()
        if reuse_MRQuery == 'new':
            mrquery.Do_MRQuery(self.topdatadir, self.basename, reuse_MRQuery,
                   self.topdirs, excludes)
        else:
            mrquery.Do_MRQuery(self.topdatadir, self.basename, reuse_MRQuery)

    def do_haar(self, reuse_MRQuery):
        import haar
        print >> sys.stderr, 'Haar'
        if reuse_MRQuery == 'new':
            hobj = haar.Haar(self.topdatadir, self.basename, self.topdirs,
                    list())
        else:
            hobj = haar.Haar(self.topdatadir, self.basename, reuse_MRQuery)
        hobj.mainloop()

    def do_scorearr(self, reuse_sort, usewts=True):
        import scorearr
        print >> sys.stderr, 'Scorearr'
        scorearr.Scorearr(self.topdatadir, self.basename, reuse_sort, usewts)

    def do_printscores(self, which_sortlist, common=1):
        import scorearr
        print >> sys.stderr, 'PrintScores'
        scorearr.PrintScores(self.topdatadir, self.basename,
                      which_sortlist, common)

    def do_DDL(self, distance=None, direction=None, level=None):
        import DDL
        print >> sys.stderr, 'DDL'
        DDL.DistanceDirectionLevel(self.topdatadir, self.basename,
             distance, direction, level)

    def do_html(self, use_html_piece='normal'):
        import html
        print >> sys.stderr, 'html'
        html.HTML(self.topdatadir, self.basename, use_html_piece)

def new(basename, topdirs, excludes=None):
    doit = TopLevel(basename, topdirs)
    doit.do_mrquery('new', excludes)
    doit.do_haar('new')
    doit.do_scorearr('new')

def update(basename):
    doit = TopLevel(basename)
    doit.do_mrquery('update')
    doit.do_haar('update')
    doit.do_scorearr('continue')

def DDLhtml(basename, local_matches=True, use_html_piece='normal'):
    doit = TopLevel(basename)
    if local_matches:
        doit.do_DDL()
    else:
        # Exclude matches between two images in the same directory.
        doit.do_DDL('far', 'upwards', 1)
    doit.do_html(use_html_piece)

def singleton(basename, which, topdirs=None, local_matches=True,
             use_html_piece='normal'):
    if which not in ('new', 'update', 'html'):
        raise ValueError("\"which\" must be 'new', 'update' or 'html'")
    if which == 'new':
        new(basename, topdirs)
    if which == 'update':
        update(basename)
    DDLhtml(basename, local_matches, use_html_piece)

if __name__ == '__main__':
    # "topdir" is a directory. The images under this directory will be
    # processed.  The "basename" is used in many places. It is the name used
    # for this run of the mrquery programs.
    usage = './mrq.py [options] <basename> <new/update/html> [<topdir> ...]\n' \
      '        if "new" is given, topdir(s) must be given.'
    parser = optparse.OptionParser(usage=usage)
    parser.add_option('-m', '--no-local-matches', default=False,
         action='store_true',
         help='Ignore any match where the two images are in the same directory.')
    parser.add_option('-f', '--flat', default=False, action='store_true',
         help='If this option is given, the output html will be ' \
              'a flat list of matches. The default is to arrange the html ' \
              'by pairs of directories.')
    options, args = parser.parse_args()
    local_matches = not options.no_local_matches
    if options.flat:
        use_html_piece = 'flat'
    else:
        use_html_piece = 'normal'

    if len(args) not in (2, 3):
        parser.print_help()
        sys.exit(2)
    if args[1] == 'new' and len(args) == 2:
        sys.exit('topdir(s) must be given when creating a new mrquery database. Try "./mrq.py -h".')
    if args[1] != 'new' and len(args) >= 3:
        sys.exit('Do not give topdirs for "update" or "html" operation. Try "./mrq.py -h".')
    basename = args[0]
    which = args[1]
    if len(args) == 2:
       topdirs = None
    else:
       topdirs = args[2:]
    singleton(basename, which, topdirs, local_matches, use_html_piece)
