#! /usr/bin/env python

"""Sets pairs of directories to be ignored when html output is created.

The user edits the output of "getXdirs.py" and inputs it here. See
"getXdirs.py" for further documentation.
"""
"""
Copyright 2006 Edward C. Jones

This file is part of mrquery.

mrquery is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

mrquery is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
mrquery; if not, write to the Free Software Foundation, Inc., 51 Franklin
St, Fifth Floor, Boston, MA 02110-1301 USA
"""
import sys, optparse
import constants, pathnames, mrquery, html
from LN import LN, MLLN

def getname(line):
    isXed = line.startswith('X ')
    line = line[1:].strip()
    iden_code, num, score, name = line.split()
    is_iden = (iden_code == 'I')
    return name, isXed, is_iden

usage = 'USAGE: ./setignores.py <input>'
parser = optparse.OptionParser(usage=usage)
options, args = parser.parse_args()
if len(args) != 1:
    parser.print_help()
    sys.exit(2)
filein = args[0]

text = open(filein, 'r').read().strip()
out = list()
text = text.strip()
lines = text.splitlines()
# The first three lines look like:
#     basename
#     X Do nothing.
#       Create html for all directory pairs (reinitialize mrq.html_ignores).
basename = lines[0].strip()
out.append('# ' + basename)
words = lines[1].split()
donothing = (len(words) == 3 and words[0] == 'X' and words[1] == 'Do')

words = lines[2].split()
reinitialize = (len(words) >= 2 and words[0] == 'X' and words[1] == 'Create')

del lines[:4]

if donothing and reinitialize:
   raise TypeError('if "Do Nothing" is X\'ed, then the "Create html" '
        'line must not be X\'ed')

if donothing:
    LN('There is an "X" in front of "Do nothing". Nothing done.')
    sys.exit()
mrq = mrquery.Retrieve_MRQuery(constants.TOPDATADIR, basename)
if reinitialize:
    mrq.html_ignores = list()
    mrq.Store()
    sys.exit()
ignores = list()
for line in lines:
    words = line.split()
    if len(words) > 2 or (len(words) == 2 and words[0] != 'X'):
        raise ValueError('lines must be of the form "<directory>" or '
                 '"X <directory>"')
    if words[0] == 'X':
        ignores.append(words[1])
ignores.sort()
# If a directory is in "ignores", then directories under it need not be.
for i in reversed(range(1, len(ignores))):
    if pathnames.isunder(ignores[i], ignores[i-1]):
        del ignores[i]
mrq.html_ignores = ignores
mrq.Store()
